#include "Decimator.h"


CDecimator::CDecimator(void)
{
}


CDecimator::~CDecimator(void)
{
}


void CDecimator::decimateSamples(float* pLeftDeciBuffer, float* pRightDeciBuffer,float& ynL, float& ynR)
{
	if(!pLeftDeciBuffer || !pRightDeciBuffer)
		return;

	for(int i=0; i<m_nL; i++)
	{
		// decimate next sample
		//
		// ynL and ynR are valid (and returned) after the last call to decimateNextOutputSample()
		this->decimateNextOutputSample(pLeftDeciBuffer[i], pRightDeciBuffer[i], ynL, ynR);
	}
}

bool CDecimator::decimateNextOutputSample(float xnL, float xnR, float& fLeftOutput, float& fRightOutput)
{
	// read inputs
	//
	// left
	m_pLeftInputBuffer[m_nWriteIndex] = xnL; 

	// right
	m_pRightInputBuffer[m_nWriteIndex] = xnR; 

	// OPTIMIZE THIS: Can skip L-1 convolutions! 

	// reset: read index for Delay Line -> write index
	m_nReadIndexDL = m_nWriteIndex;

	// reset: read index for IR - > top (0)
	m_nReadIndexH = 0;

	// accumulator
	float yn_accumL = 0;	
	float yn_accumR = 0;	

	// convolve:
	for(int i=0; i<m_nIRLength; i++)
	{
		// do the sum of products
		yn_accumL += m_pLeftInputBuffer[m_nReadIndexDL]*m_pIRBuffer[m_nReadIndexH];
		yn_accumR += m_pRightInputBuffer[m_nReadIndexDL]*m_pIRBuffer[m_nReadIndexH];

		// advance the IR index
		m_nReadIndexH++;

		// decrement the Delay Line index
		m_nReadIndexDL--;

		// check for wrap of delay line (no need to check IR buffer)
		if(m_nReadIndexDL < 0)
			m_nReadIndexDL = m_nIRLength -1;
	}

	// write out
	fLeftOutput = yn_accumL;

	// write out
	fRightOutput = yn_accumR; 

	// incremnent the pointers and wrap if necessaryx
	m_nWriteIndex++;
	if(m_nWriteIndex >= m_nIRLength)
		m_nWriteIndex = 0;

	return true;
}


